package com.usky.iot.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.iot.domain.DmpDeviceInfo;
import com.usky.iot.mapper.DmpDeviceInfoMapper;
import com.usky.iot.service.DmpDeviceInfoService;
import com.usky.iot.service.vo.DmpDeviceInfoRequest;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;

/**
 * <p>
 * 设备信息表 服务实现类
 * </p>
 *
 * @author ya
 * @since 2022-10-08
 */
@Service
public class DmpDeviceInfoServiceImpl extends AbstractCrudService<DmpDeviceInfoMapper, DmpDeviceInfo> implements DmpDeviceInfoService {

    @Override
    public boolean add(DmpDeviceInfo dmpDeviceInfo) {
        if (checkNameUnique(dmpDeviceInfo)){
            throw new BusinessException("新增设备信息'" + dmpDeviceInfo.getDeviceName() + "'失败，设备信息已存在");
        }
        return this.save(dmpDeviceInfo);
    }

    @Override
    public void update(DmpDeviceInfo dmpDeviceInfo) {
        if (checkNameUnique(dmpDeviceInfo)){
            throw new BusinessException("修改设备信息'" + dmpDeviceInfo.getDeviceName() + "'失败，设备信息已存在");
        }
        this.updateById(dmpDeviceInfo);
    }

    @Override
    public boolean checkNameUnique(DmpDeviceInfo dmpDeviceInfo) {
        Integer id = null == dmpDeviceInfo.getId() ? -1 : dmpDeviceInfo.getId();
        LambdaQueryWrapper<DmpDeviceInfo> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper
                .eq(DmpDeviceInfo::getDeviceName,dmpDeviceInfo.getDeviceName())
                .eq(DmpDeviceInfo::getProductId,dmpDeviceInfo.getProductId())
                .eq(DmpDeviceInfo::getDeleteFlag,0);
        DmpDeviceInfo one = this.getOne(queryWrapper);
        return null != one && !Objects.equals(one.getId(), id);
    }

    @Override
    public CommonPage<DmpDeviceInfo> page(DmpDeviceInfoRequest diRequest) {
        IPage<DmpDeviceInfo> page = new Page<>(diRequest.getCurrent(), diRequest.getSize());
        page = baseMapper.page(page,diRequest);
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getCurrent(),page.getSize());
    }

    @Override
    public boolean remove(Integer id) {
        DmpDeviceInfo dmpDeviceInfo = this.getById(id);
        Optional.ofNullable(dmpDeviceInfo).orElseThrow(() -> new BusinessException("设备不存在"));
        dmpDeviceInfo.setDeleteFlag(1);
        return this.updateById(dmpDeviceInfo);
    }

    @Override
    public List<Map<String, Object>> deviceCollect(List<Integer> productIds) {
        QueryWrapper<DmpDeviceInfo> query = Wrappers.query();
        query.select("product_id as productId","count(*) as count")
                .in(CollectionUtil.isNotEmpty(productIds),"product_id",productIds)
                .groupBy("product_id");
        return this.listMaps(query);
    }

    @Override
    public int deviceCountByProductId(Integer productId) {
        LambdaQueryWrapper<DmpDeviceInfo> lambdaQuery = Wrappers.lambdaQuery();
        lambdaQuery.eq(DmpDeviceInfo::getProductId,productId)
                .eq(DmpDeviceInfo::getDeleteFlag,0);
        return this.count(lambdaQuery);
    }
}
